@@ -2,9 +2,8 @@ package AnyEvent::WebSocket::Client;
 
 use strict;
 use warnings;
-use v5.10;
 use Moo;
-use warnings NONFATAL => 'all';
+use AE;
 use AnyEvent;
 use AnyEvent::Handle;
 use AnyEvent::Socket qw( tcp_connect );
@@ -13,7 +12,7 @@ use AnyEvent::WebSocket::Connection;
 use PerlX::Maybe qw( maybe provided );
 
 # ABSTRACT: WebSocket client for AnyEvent
-our $VERSION = '0.15_01'; # VERSION
+our $VERSION = '0.33_01'; # TRIAL VERSION
 
 
 has timeout => (
@@ -41,7 +40,7 @@ sub connect
     $uri = URI->new($uri);
   }
   
-  my $done = AnyEvent->condvar;
+  my $done = AE::cv;
 
   # TODO: should we also accept http and https URLs?
   # probably.
@@ -93,7 +92,7 @@ sub connect
       elsif($handshake->is_done)
       {
         undef $handshake;
-        $done->send(AnyEvent::WebSocket::Connection->new(handle => $hdl));
+        $done->send(AnyEvent::WebSocket::Connection->new(handle => $hdl, masked => 1));
         undef $hdl;
         undef $done;
       }
@@ -108,13 +107,15 @@ __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 AnyEvent::WebSocket::Client - WebSocket client for AnyEvent
 
 =head1 VERSION
 
-version 0.15_01
+version 0.33_01
 
 =head1 SYNOPSIS
 
@@ -123,9 +124,16 @@ version 0.15_01
  my $client = AnyEvent::WebSocket::Client->new;
  
  $client->connect("ws://localhost:1234/service")->cb(sub {
-   my $connection = eval { shift->recv };
+ 
+   # make $connection an our variable rather than
+   # my so that it will stick around.  Once the
+   # connection falls out of scope any callbacks
+   # tied to it will be destroyed.
+   our $connection = eval { shift->recv };
    if($@) {
      # handle error...
+     warn $@;
+     return;
    }
    
    # send a message through the websocket...
@@ -152,6 +160,11 @@ version 0.15_01
  
  });
 
+ ## uncomment to enter the event loop before exiting.
+ ## Note that calling recv on a condition variable before
+ ## it has been triggered does not work on all event loops
+ #AnyEvent->condvar->recv;
+
 =head1 DESCRIPTION
 
 This class provides an interface to interact with a web server that provides
@@ -162,8 +175,8 @@ control, but if that is not necessary then this class may save you some time.
 
 The recommended API was added to the L<AnyEvent::WebSocket::Connection>
 class with version 0.12, so it is recommended that you include that version
-when using this module.  The older API will continue to work for now with
-deprecation warnings.
+when using this module.  The older version of the API has since been
+deprecated and removed.
 
 =head1 ATTRIBUTES
 
@@ -184,7 +197,9 @@ SSL/TLS verification.
 
 =head1 METHODS
 
-=head2 $client-E<gt>connect($uri)
+=head2 connect
+
+ my $cv = $client->connect($uri)
 
 Open a connection to the web server and open a WebSocket to the resource
 defined by the given URL.  The URL may be either an instance of L<URI::ws>,
@@ -196,6 +211,41 @@ be either an instance of L<AnyEvent::WebSocket::Connection> or a croak
 message indicating a failure.  The synopsis above shows how to catch
 such errors using C<eval>.
 
+=head1 FAQ
+
+=head2 My program exits before doing anything, what is up with that?
+
+See this FAQ from L<AnyEvent>: 
+L<AnyEvent::FAQ#My-program-exits-before-doing-anything-whats-going-on>.
+
+It is probably also a good idea to review the L<AnyEvent> documentation
+if you are new to L<AnyEvent> or event-based programming.
+
+=head2 My callbacks aren't being called!
+
+Make sure that the connection object is still in scope.  This often happens
+if you use a C<my $connection> variable and don't save it somewhere.  For
+example:
+
+ $client->connect("ws://foo/service")->cb(sub {
+ 
+   my $connection = eval { shift->recv };
+   
+   if($@)
+   {
+     warn $@;
+     return;
+   }
+   
+   ...
+ });
+
+Unless C<$connection> is saved somewhere it will get deallocated along with
+any associated message callbacks will also get deallocated once the connect
+callback is executed.  One way to make sure that the connection doesn't
+get deallocated is to make it a C<our> variable (as in the synopsis above)
+instead.
+
 =head1 CAVEATS
 
 This is pretty simple minded and there are probably WebSocket features
@@ -216,6 +266,10 @@ L<AnyEvent::WebSocket::Message>
 
 =item *
 
+L<AnyEvent::WebSocket::Server>
+
+=item *
+
 L<AnyEvent>
 
 =item *
@@ -232,17 +286,23 @@ L<Protocol::WebSocket>
 
 =item *
 
+L<Net::WebSocket::Server>
+
+=item *
+
+L<Net::Async::WebSocket>
+
+=item *
+
 L<RFC 6455 The WebSocket Protocol|http://tools.ietf.org/html/rfc6455>
 
 =back
 
-=cut
-
 =head1 AUTHOR
 
-author: Graham Ollis <plicease@cpan.org>
+Author: Graham Ollis E<lt>plicease@cpan.orgE<gt>
 
-contributors:
+Contributors:
 
 Toshio Ito
 
