@@ -2,17 +2,17 @@ package AnyEvent::WebSocket::Connection;
 
 use strict;
 use warnings;
-use v5.10;
 use Moo;
-use warnings NONFATAL => 'all';
 use Protocol::WebSocket::Frame;
 use Scalar::Util qw( weaken );
 use Encode ();
+use AE;
+use AnyEvent;
 use AnyEvent::WebSocket::Message;
 use Carp qw( croak carp );
 
 # ABSTRACT: WebSocket connection for AnyEvent
-our $VERSION = '0.15_01'; # VERSION
+our $VERSION = '0.33_01'; # TRIAL VERSION
 
 
 has handle => (
@@ -20,6 +20,12 @@ has handle => (
   required => 1,
 );
 
+
+has masked => (
+  is      => 'ro',
+  default => sub { 0 },
+);
+
 foreach my $type (qw( each_message next_message finish ))
 {
   has "_${type}_cb" => (
@@ -29,35 +35,94 @@ foreach my $type (qw( each_message next_message finish ))
   );
 }
 
+foreach my $flag (qw( _is_read_open _is_write_open ))
+{
+  has $flag => (
+    is       => 'rw',
+    init_arg => undef,
+    default  => sub { 1 },
+  );
+}
+
+has "_is_finished" => (
+  is       => 'rw',
+  init_arg => undef,
+  default  => sub { 0 },
+);
+
 sub BUILD
 {
   my $self = shift;
   weaken $self;
   my $finish = sub {
+    my $strong_self = $self; # preserve $self because otherwise $self can be destroyed in the callbacks.
+    return if $self->_is_finished;
+    $self->_is_finished(1);
+    $self->handle->push_shutdown;
+    $self->_is_read_open(0);
+    $self->_is_write_open(0);
     $_->($self) for @{ $self->_finish_cb };
   };
   $self->handle->on_error($finish);
   $self->handle->on_eof($finish);
 
   my $frame = Protocol::WebSocket::Frame->new;
-  
-  $self->handle->on_read(sub {
-    $frame->append($_[0]{rbuf});
-    while(defined(my $body = $frame->next_bytes))
+
+  my $read_cb = sub {
+    my ($handle) = @_;
+    local $@;
+    my $strong_self = $self; # preserve $self because otherwise $self can be destroyed in the callbacks
+    my $success = eval
     {
-      if($frame->is_text || $frame->is_binary)
+      $frame->append($handle->{rbuf});
+      while(defined(my $body = $frame->next_bytes))
       {
-        my $message = AnyEvent::WebSocket::Message->new(
-          body   => $body,
-          opcode => $frame->opcode,
-        );
-      
-        $_->($self, $message) for @{ $self->_next_message_cb };
-        @{ $self->_next_message_cb } = ();
-        $_->($self, $message) for @{ $self->_each_message_cb };
+        next if !$self->_is_read_open; # not 'last' but 'next' in order to consume data in $frame buffer.
+        if($frame->is_text || $frame->is_binary)
+        {
+          my $message = AnyEvent::WebSocket::Message->new(
+            body   => $body,
+            opcode => $frame->opcode,
+          );
+          
+          $_->($self, $message) for @{ $self->_next_message_cb };
+          @{ $self->_next_message_cb } = ();
+          $_->($self, $message) for @{ $self->_each_message_cb };
+        }
+        elsif($frame->is_close)
+        {
+          $self->_is_read_open(0);
+          $self->close();
+        }
+        elsif($frame->is_ping)
+        {
+          $self->send(AnyEvent::WebSocket::Message->new(opcode => 10, body => $body));
+        }
       }
+      1; # succeed to parse.
+    };
+    if(!$success)
+    {
+      $self->handle->push_shutdown;
+      $self->_is_write_open(0);
+      $self->_is_read_open(0);
     }
-  });
+  };
+
+  # Delay setting on_read callback. This is necessary to make sure all
+  # received data are handled by each_message and/or next_message
+  # callbacks. If there is some data in rbuf, changing the on_read
+  # callback makes the callback fire, but there is of course no
+  # each_message/next_message callback to receive the message yet.
+  $self->handle->on_read(undef);
+  my $idle_w; $idle_w = AE::idle sub {
+    undef $idle_w;
+    if(defined($self))
+    {
+      $read_cb->($self->handle); # make sure to read remaining data in rbuf.
+      $self->handle->on_read($read_cb);
+    }
+  };
 }
 
 
@@ -65,15 +130,17 @@ sub send
 {
   my($self, $message) = @_;
   my $frame;
+  
+  return $self if !$self->_is_write_open;
+  
   if(ref $message)
   {
-    $DB::single = 1;
-    $frame = Protocol::WebSocket::Frame->new($message->body);
+    $frame = Protocol::WebSocket::Frame->new(buffer => $message->body, masked => $self->masked);
     $frame->opcode($message->opcode);
   }
   else
   {
-    $frame = Protocol::WebSocket::Frame->new($message);
+    $frame = Protocol::WebSocket::Frame->new(buffer => $message, masked => $self->masked);
   }
   $self->handle->push_write($frame->to_bytes);
   $self;
@@ -106,41 +173,19 @@ sub on
 
 sub close
 {
-  my($self) = @_;
+  my($self, $code, $reason) = @_;
 
-  $self->handle->push_write(Protocol::WebSocket::Frame->new(type => 'close')->to_bytes);
+  use Encode qw( encode );
+  my $body = pack('n', ($code) ? $code : '1005');
+
+  $body .= encode 'UTF-8', $reason if defined $reason;
+
+  $self->send(AnyEvent::WebSocket::Message->new(
+    opcode => 8,
+    body => $body,
+  ));
   $self->handle->push_shutdown;
-}
-
-
-
-sub on_each_message
-{
-  my($self, $cb) = @_;
-  carp "on_each_message is deprecated" if warnings::enabled('deprecated');
-  $self->on(each_message => sub {
-    $cb->(Encode::decode("UTF-8",pop->body));
-  });
-  $self;
-}
-
-
-sub on_next_message
-{
-  my($self, $cb) = @_;
-  carp "on_next_message is deprecated" if warnings::enabled('deprecated');
-  $self->on(next_message => sub {
-    $cb->(Encode::decode("UTF-8",pop->body));
-  });
-  $self;
-}
-
-
-sub on_finish
-{
-  my($self, $cb) = @_;
-  carp "on_finish is deprecated" if warnings::enabled('deprecated');
-  $self->on(finish => $cb);
+  $self->_is_write_open(0);
   $self;
 }
 
@@ -150,13 +195,15 @@ __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 AnyEvent::WebSocket::Connection - WebSocket connection for AnyEvent
 
 =head1 VERSION
 
-version 0.15_01
+version 0.33_01
 
 =head1 SYNOPSIS
 
@@ -183,23 +230,22 @@ version 0.15_01
  # a callback)
  $connection->close;
 
-(See L<AnyEvent::WebSocket::Client> on how to create
-a connection)
+(See L<AnyEvent::WebSocket::Client> or L<AnyEvent::WebSocket::Server> on 
+how to create a connection)
 
 =head1 DESCRIPTION
 
-This class represents a WebSocket connection with a remote
-server (or in the future perhaps a client).
+This class represents a WebSocket connection with a remote server or a 
+client.
 
-If the connection object falls out of scope then the connection
-will be closed gracefully.
+If the connection object falls out of scope then the connection will be 
+closed gracefully.
 
-This class was created for a client to connect to a server 
-via L<AnyEvent::WebSocket::Client>, but it may be useful to
-reuse it for a server to interact with a client if a
-C<AnyEvent::WebSocket::Server> is ever created (after the
-handshake is complete, the client and server look pretty
-much the same).
+This class was created for a client to connect to a server via 
+L<AnyEvent::WebSocket::Client>, and was later extended to work on the 
+server side via L<AnyEvent::WebSocket::Server>.  Once a WebSocket 
+connection is established, the API for both client and server is 
+identical.
 
 =head1 ATTRIBUTES
 
@@ -211,15 +257,25 @@ You should not use the handle directly after creating L<AnyEvent::WebSocket::Con
 
 Usually only useful for creating server connections, see below.
 
+=head2 masked
+
+If set to true, it masks outgoing frames. The default is false.
+
 =head1 METHODS
 
-=head2 $connection-E<gt>send($message)
+=head2 send
+
+ $connection->send($message);
 
 Send a message to the other side.  C<$message> may either be a string
 (in which case a text message will be sent), or an instance of
 L<AnyEvent::WebSocket::Message>.
 
-=head2 $connection-E<gt>on($event => $cb)
+=head2 on
+
+ $connection->on(each_message => $cb);
+ $connection->on(each_message => $cb);
+ $connection->on(finish => $cb);
 
 Register a callback to a particular event.
 
@@ -244,11 +300,14 @@ Called only for the next message received from the WebSocket.
 
 Called when the connection is terminated
 
-=head3
+=head2 close
 
-=head2 $connection-E<gt>close
+ $connection->close;
+ $connection->close($code);
+ $connection->close($code, $reason);
 
-Close the connection.
+Close the connection.  You may optionally provide a code and a reason.
+By default the code 1005 is used with an empty string as the reason.
 
 =head1 SERVER CONNECTIONS
 
@@ -258,31 +317,6 @@ connections.  For details, contact the author and/or take a look at the
 source for L<AnyEvent::WebSocket::Client> and the examples that come with
 L<Protocol::WebSocket>.
 
-=head1 DEPRECATED METHODS
-
-The methods in this section are deprecated and may be removed from a
-future version of this class.  They should not be used for new code,
-and are only remain documented here to aid in understanding legacy
-code that use them.
-
-=head2 $connection-E<gt>on_each_message($cb)
-
-Register a callback to be called on each subsequent message received.
-The message itself will be passed in as the only parameter to the
-callback.
-The message is a decoded text string.
-
-=head2 $connection-E<gt>on_next_message($cb)
-
-Register a callback to be called the next message received.
-The message itself will be passed in as the only parameter to the
-callback.
-The message is a decoded text string.
-
-=head2 $connection-E<gt>on_finish($cb)
-
-Register a callback to be called when the connection is closed.
-
 =head1 SEE ALSO
 
 =over 4
@@ -297,6 +331,10 @@ L<AnyEvent::WebSocket::Message>
 
 =item *
 
+L<AnyEvent::WebSocket::Server>
+
+=item *
+
 L<AnyEvent>
 
 =item *
@@ -305,13 +343,11 @@ L<RFC 6455 The WebSocket Protocol|http://tools.ietf.org/html/rfc6455>
 
 =back
 
-=cut
-
 =head1 AUTHOR
 
-author: Graham Ollis <plicease@cpan.org>
+Author: Graham Ollis E<lt>plicease@cpan.orgE<gt>
 
-contributors:
+Contributors:
 
 Toshio Ito
 
