@@ -2,13 +2,11 @@ package AnyEvent::WebSocket::Message;
 
 use strict;
 use warnings;
-use v5.10;
 use Moo;
-use warnings NONFATAL => 'all';
 use Encode ();
 
 # ABSTRACT: WebSocket message for AnyEvent
-our $VERSION = '0.15_01'; # VERSION
+our $VERSION = '0.33_01'; # TRIAL VERSION
 
 
 has body => ( is => 'ro', required => 1 );
@@ -33,13 +31,15 @@ __END__
 
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 AnyEvent::WebSocket::Message - WebSocket message for AnyEvent
 
 =head1 VERSION
 
-version 0.15_01
+version 0.33_01
 
 =head1 SYNOPSIS
 
@@ -72,27 +72,39 @@ The integer code for the type of message.
 
 =head1 METHODS
 
-=head2 $message-E<gt>decoded_body
+=head2 decoded_body
+
+ my $body = $message->decoded_body;
 
 Returns the body decoded from UTF-8.
 
-=head2 $message-E<gt>is_text
+=head2 is_text
+
+ my $bool = $message->is_text;
 
 True if the message is text.
 
-=head2 $message-E<gt>is_binary
+=head2 is_binary
+
+ my $bool = $message->is_binary;
 
 True if the message is binary.
 
-=head2 $message-E<gt>is_close
+=head2 is_close
+
+ my $bool = $message->is_close;
 
 True if the message is a close message.
 
-=head2 $message-E<gt>is_ping
+=head2 is_ping
+
+ my $bool = $message->is_ping
 
 True if the message is a ping.
 
-=head2 $message-E<gt>is_pong
+=head2 is_pong
+
+ my $bool = $message->is_pong;
 
 True if the message is a pong.
 
@@ -110,6 +122,10 @@ L<AnyEvent::WebSocket::Connection>
 
 =item *
 
+L<AnyEvent::WebSocket::Server>
+
+=item *
+
 L<AnyEvent>
 
 =item *
@@ -118,13 +134,11 @@ L<RFC 6455 The WebSocket Protocol|http://tools.ietf.org/html/rfc6455>
 
 =back
 
-=cut
-
 =head1 AUTHOR
 
-author: Graham Ollis <plicease@cpan.org>
+Author: Graham Ollis E<lt>plicease@cpan.orgE<gt>
 
-contributors:
+Contributors:
 
 Toshio Ito
 
